@@ -1,8 +1,8 @@
 /*****************************************************************************
- * VLCLocalServerListViewController.m
+ * VLCServerListViewController.m
  * VLC for iOS
  *****************************************************************************
- * Copyright (c) 2013-2018 VideoLAN. All rights reserved.
+ * Copyright (c) 2013-2020 VideoLAN. All rights reserved.
  * $Id$
  *
  * Authors: Felix Paul Kühne <fkuehne # videolan.org>
@@ -40,6 +40,9 @@
 #import "VLCWiFiUploadTableViewCell.h"
 
 #import "VLCBoxController.h"
+#import <OneDriveSDK.h>
+#import "VLCOneDriveConstants.h"
+#import "VLCDropboxConstants.h"
 
 #import "VLC-Swift.h"
 
@@ -64,14 +67,8 @@
 - (instancetype)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
 {
     self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
-
     if (self) {
         [self setupUI];
-
-        // Start Box session on init to check whether it is logged in or not as soon as possible
-        [[VLCBoxController sharedInstance] startSession];
-        // Request directory listing to check authorization
-        [[VLCBoxController sharedInstance] requestDirectoryListingAtPath:nil];
     }
     return self;
 }
@@ -79,6 +76,7 @@
 - (void)loadView
 {
     [super loadView];
+    [self configureCloudControllers];
 
     _scrollView = [[UIScrollView alloc] init];
     _scrollView.translatesAutoresizingMaskIntoConstraints = NO;
@@ -174,9 +172,15 @@
 {
     [super viewDidLoad];
 
-    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(themeDidChange) name:kVLCThemeDidChangeNotification object:nil];
-    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(contentSizeDidChange) name:UIContentSizeCategoryDidChangeNotification object:nil];
-    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(boxSessionUpdated) name:VLCBoxControllerSessionUpdated object:nil];
+    NSNotificationCenter *notificationCenter = [NSNotificationCenter defaultCenter];
+    [notificationCenter addObserver:self selector:@selector(themeDidChange) name:kVLCThemeDidChangeNotification object:nil];
+    [notificationCenter addObserver:self selector:@selector(contentSizeDidChange) name:UIContentSizeCategoryDidChangeNotification object:nil];
+    [notificationCenter addObserver:self selector:@selector(boxSessionUpdated) name:VLCBoxControllerSessionUpdated object:nil];
+    [notificationCenter addObserver:self selector:@selector(miniPlayerIsShown)
+                               name:VLCPlayerDisplayControllerDisplayMiniPlayer object:nil];
+    [notificationCenter addObserver:self selector:@selector(miniPlayerIsHidden)
+                               name:VLCPlayerDisplayControllerHideMiniPlayer object:nil];
+
     [self themeDidChange];
     NSArray *browserClasses = @[
                                 [VLCLocalNetworkServiceBrowserUPnP class],
@@ -209,9 +213,22 @@
     if (@available(iOS 11.0, *)) {
         self.navigationController.navigationBar.prefersLargeTitles = YES;
     }
+    VLCPlaybackService.sharedInstance.playerDisplayController.isMiniPlayerVisible
+    ? [self miniPlayerIsShown] : [self miniPlayerIsHidden];
     [_remoteNetworkTableView reloadData];
 }
 
+- (void)miniPlayerIsShown
+{
+    _localNetworkTableView.contentInset = UIEdgeInsetsMake(0, 0,
+                                                           VLCAudioMiniPlayer.height, 0);
+}
+
+- (void)miniPlayerIsHidden
+{
+    _localNetworkTableView.contentInset = UIEdgeInsetsMake(0, 0, 0, 0);
+}
+
 - (BOOL)shouldAutorotate
 {
     UIInterfaceOrientation toInterfaceOrientation = [[UIApplication sharedApplication] statusBarOrientation];
@@ -243,6 +260,21 @@
         loginViewController.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:NSLocalizedString(@"BUTTON_CANCEL", nil) style:UIBarButtonItemStylePlain target:self action:@selector(_dismissLogin)];
 }
 
+- (void)configureCloudControllers
+{
+    VLCBoxController *boxController = [VLCBoxController sharedInstance];
+    // Start Box session on init to check whether it is logged in or not as soon as possible
+    [boxController startSession];
+    // Request directory listing to check authorization
+    [boxController requestDirectoryListingAtPath:nil];
+
+    // Configure Dropbox
+    [DBClientsManager setupWithAppKey:kVLCDropboxAppKey];
+
+    // Configure OneDrive
+    [ODClient setMicrosoftAccountAppId:kVLCOneDriveClientID scopes:@[@"onedrive.readwrite", @"offline_access"]];
+}
+
 - (void)boxSessionUpdated
 {
     __weak typeof(self) weakSelf = self;
@@ -267,8 +299,9 @@
 
 - (void)tableView:(UITableView *)tableView willDisplayCell:(VLCNetworkListCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath
 {
-    cell.titleLabel.textColor = cell.folderTitleLabel.textColor = cell.thumbnailView.tintColor = PresentationTheme.current.colors.cellTextColor;
-    cell.subtitleLabel.textColor = PresentationTheme.current.colors.cellDetailTextColor;
+    ColorPalette *themeColors = PresentationTheme.current.colors;
+    cell.titleLabel.textColor = cell.folderTitleLabel.textColor = cell.thumbnailView.tintColor = themeColors.cellTextColor;
+    cell.subtitleLabel.textColor = themeColors.cellDetailTextColor;
 }
 
 - (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
@@ -340,7 +373,6 @@
                                                                     target:self
                                                                     action:@selector(_dismissLogin)];
         }
-
     } else {
         [self.navigationController pushViewController:loginViewController animated:YES];
     }
@@ -361,8 +393,9 @@
     _refreshControl.backgroundColor = PresentationTheme.current.colors.background;
     self.navigationController.view.backgroundColor = PresentationTheme.current.colors.background;
     if (@available(iOS 13.0, *)) {
-        self.navigationController.navigationBar.standardAppearance = [VLCApperanceManager navigationbarAppearance];
-        self.navigationController.navigationBar.scrollEdgeAppearance = [VLCApperanceManager navigationbarAppearance];
+        UINavigationBarAppearance *navigationBarAppearance = [VLCAppearanceManager navigationbarAppearance];
+        self.navigationController.navigationBar.standardAppearance = navigationBarAppearance;
+        self.navigationController.navigationBar.scrollEdgeAppearance = navigationBarAppearance;
     }
     [self setNeedsStatusBarAppearanceUpdate];
 }
