@@ -32,6 +32,8 @@
 
 - (void)moveFileFrom:(NSString *)filepath;
 - (void)cleanCache;
+#if TARGET_OS_IOS
 - (void)resetIdleTimer;
+#endif
 
 @end
